def isYear(part):
	try:
		if 1990 < int(part) < 2100:
			return True
		if 19900000 < int(part) < 21000000:
			return True
	except:
		...
	return False

def containYear(link):
	for part in link.split('/'):
		if isYear(part):
			return True
	return False

def hasSignature(link, sig_len):
	for length in sig_len:
		if link[-length:][:1] == '-':
			return True
	return False


from bs4 import BeautifulSoup
import cached_url
from telegram_util import matchKey

offtopic_tags = ['nav', 'footer', 'aside', 'header']

def getSoup(site):
	soup = BeautifulSoup(cached_url.get(site), 'html.parser')
	for item in soup.find_all('a', rel='author'):
		item.decompose()
	for tag in offtopic_tags:
		for item in soup.find_all(tag):
			item.decompose()
	if 'freewechat.com' in site:
		for item in soup.find_all('div', id='hot-articles'):
			item.decompose()
	return soup


from telegram_util import matchKey

def getDoubanId(link):
	if not matchKey(link, ['note', 'group/topic', 'status', 'album']):
		return
	if matchKey(link, ['notes', 'statuses']):
		return
	if 'http' not in link:
		return
	parts = link.split('/')
	for part in parts[:-1]:
		try:
			int(part)
			return part
		except:
			...

def countLike(link, soup):
	douban_id = getDoubanId(link)
	result = 0
	for item in soup.find_all():
		if item.attrs and douban_id in str(item.attrs):
			result += int(item.get('data-count') or 0)
			try:
				result += int(item.parent.parent.find('td', class_='r-count').text)
			except:
				...
	return result

def getLimit(site):
	if '/explore' in site:
		return 120
	if '/blabla' in site: # 鹅组
		return 200
	if '/group/' in site:
		return 200
	return -1

def getDoubanLinks(site, links, soup):
	counted_items = [(countLike(link, soup), link) for link in links
		if getDoubanId(link)]
	counted_items.sort(reverse=True)
	limit = getLimit(site)
	return [item[1] for item in counted_items if item[0] > limit]


import cached_url
import yaml

def getVocusLinks(site):
	pid = site.split('.cc/')[1].split('/')[0]
	api_link = 'https://api.sosreader.com/api/articles?publicationId=' + pid
	content = yaml.load(cached_url.get(api_link), Loader=yaml.FullLoader)
	return ['https://vocus.cc/' + pid + '/' + article.get('_id') for 
		article in content.get('articles')]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

name = 'link_extractor'

from telegram_util import matchKey
from .util import containYear, hasSignature
from .get_soup import getSoup
from .douban import getDoubanLinks
from .vocus import getVocusLinks
from .douban_notes import getDoubanNotes
from .matters import filterMatters
from collections import OrderedDict
import pkg_resources
import yaml

config = pkg_resources.resource_string(__name__, 'config.yaml')
config = yaml.load(config, Loader=yaml.FullLoader)

def yieldLinks(soup):
	for note in soup.find_all('div', class_='note-container'): # douban notes
		yield note.get('data-url'), note
	for item in soup.find_all('a', class_='top-story'): # bbc sorting
		yield item.get('href'), item
	for container in soup.find_all(): # bbc china sorting
		if container.attrs and 'Headline' in str(container.attrs.get('class')):
			for item in container.find_all('a'):
				yield item.get('href'), item
	for item in soup.find_all('a'):
		if 'newsDetail_forward' in str(item) and 'tiptitleImg' not in str(item):
			continue
		yield item.get('href'), item
	for item in soup.find_all('div'):
		yield item.get('data-link'), item

def formatRawLink(link_package, domain, site):
	if not link_package:
		return
	link, item = link_package
	if not link:
		return
	if domain.startswith('cn.'):
		domain = domain[3:]
	link = link.strip().lstrip('/')
	for char in '#?':
		link = link.split(char)[0]
	if domain not in link and 'http' in link:
		return
	parts = set(link.split('/'))
	for key, sub_config in config.items():
		if key in site:
			must_contain_part = sub_config.get('must_contain_part')
			if must_contain_part and must_contain_part not in parts:
				return
			not_contain = sub_config.get('not_contain')
			if not_contain and matchKey(str(item), not_contain):
				return
			sig_len = sub_config.get('sig_len')
			if sig_len and not hasSignature(link, sig_len):
				return
			must_contain = sub_config.get('must_contain')
			if must_contain and must_contain not in link:
				return
	return link

def getSig(link):
	parts = link.split('/')
	sig = [len(parts), 'http' in link, containYear(link)]
	if sig[2]:
		return tuple([0] + sig)
	if (sig[1] and sig[0] > 4) or sig[0] > 1:
		return tuple([1] + sig)
	return tuple([2] + sig)

def getPreferedSig(links, site):
	dic = {}
	for link in links:
		sig = getSig(link)
		if sig in dic:
			dic[sig].append(link)
		else:
			dic[sig] = [link]
	bucket = [(len(dic[sig]), sig) for sig in dic]
	bucket.sort(reverse=True)
	for target in range(2):
		for size, sig in bucket:
			if sig[0] == target and size > 2:
				return sig
	for size, sig in bucket:
		return sig
	return (0, False, False, False) # shouldn't be here

def sigMatch(sigModel, sig):
	return sigModel[:3] == sig[:3] and ((not sigModel[3]) or sig[3])

def populateDomain(link, site):
	if 'http' in link:
		return link
	if link.startswith('zhuanlan.zhihu.com'):
		return 'https://' + link
	return '/'.join(site.split('/')[:3] + [link])

def getLinks(site):
	if 'vocus.cc' in site:
		return getVocusLinks(site)
	if 'douban.' in site and '/notes' in site:
		return list(getDoubanNotes(site.split('/')[4]))
	soup = getSoup(site)
	links = list(yieldLinks(soup))
	links = [formatRawLink(link, site.split('/')[2], site) for link in links]
	# dedup, keep order
	links = [link for link in OrderedDict.fromkeys(links) if link]
	if '.douban.' in site:
		return list(getDoubanLinks(site, links, soup))
	prefered_sig = getPreferedSig(links, site)
	links = [populateDomain(link, site) for 
		link in links if sigMatch(prefered_sig, getSig(link))]
	if 'cn.nytimes.com/opinion' in site:
		return links[:3]
	if 'gonghao51' in site:
		return links[:30] 
	if site.endswith('matters.news'):
		return list(filterMatters(links))
	return links


import yaml
import cached_url

def getDoubanNotes(uid):
	link = 'https://m.douban.com/rexxar/api/v2/user/%s/notes?start=0&count=20' % uid
	json = yaml.load(cached_url.get(link, headers={'referer': 'https://m.douban.com'}), Loader=yaml.FullLoader)
	for note_obj in json['notes']:
		yield note_obj['url'].replace('\/', '/')



from bs4 import BeautifulSoup
import cached_url

def getLikes(link):
	soup = BeautifulSoup(cached_url.get(link), 'html.parser')
	item = soup.find('span', class_="clap").nextSibling
	if item:
		return int(item.text)
	else:
		return 0

def filterMatters(links):
	for link in links:
		if getLikes(link) > 300:
			yield link
